@@ -199,8 +199,10 @@
     if ([service respondsToSelector:@selector(directPlaybackURL)]) {
         NSURL *playbackURL = [service directPlaybackURL];
         if (playbackURL) {
-            VLCPlaybackController *vpc = [VLCPlaybackController sharedInstance];
-            [vpc playURL:playbackURL successCallback:nil errorCallback:nil];
+
+            VLCMediaList *medialist = [[VLCMediaList alloc] init];
+            [medialist addMedia:[VLCMedia mediaWithURL:playbackURL]];
+            [[VLCPlaybackController sharedInstance] playMediaList:medialist firstIndex:0 subtitlesFilePath:nil];
             return;
         }
     }
@@ -217,7 +219,7 @@
     loginViewController.loginInformation = login;
     loginViewController.delegate = self;
     if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
-        UINavigationController *navCon = [[VLCNavigationController alloc] initWithRootViewController:loginViewController];
+        UINavigationController *navCon = [[UINavigationController alloc] initWithRootViewController:loginViewController];
         navCon.navigationBarHidden = NO;
         navCon.modalPresentationStyle = UIModalPresentationFormSheet;
         [self presentViewController:navCon animated:YES completion:nil];
