@@ -72,6 +72,7 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     UIBarButtonItem *_shareBarButtonItem;
     UIBarButtonItem *_removeFromFolderBarButtonItem;
     UIBarButtonItem *_deleteSelectedBarButtonItem;
+    UIBarButtonItem *_rendererBarButton;
     
     NSObject *dragAndDropManager;
 }
@@ -131,13 +132,14 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
         _tableView.separatorColor = [UIColor VLCDarkBackgroundColor];
         _tableView.delegate = self;
         _tableView.dataSource = self;
+        _tableView.indicatorStyle = UIScrollViewIndicatorStyleWhite;
+        _tableView.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
         if (@available(iOS 11.0, *)) {
             _tableView.dragDelegate = ((VLCDragAndDropManager *)dragAndDropManager);
             _tableView.dropDelegate = ((VLCDragAndDropManager *)dragAndDropManager);
+        } else {
+            _tableView.tableHeaderView = _searchController.searchBar;
         }
-        _tableView.indicatorStyle = UIScrollViewIndicatorStyleWhite;
-        _tableView.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
-        _tableView.tableHeaderView = _searchController.searchBar;
         UINib *nib = [UINib nibWithNibName:@"VLCPlaylistTableViewCell" bundle:nil];
         [_tableView registerNib:nib forCellReuseIdentifier:kPlaylistCellIdentifier];
     }
@@ -166,9 +168,13 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     [_collectionView reloadData];
     if (self.usingTableViewToShowData) {
         [contentView addSubview:_tableView];
+        if (@available(iOS 11.0, *))
+            [self setSearchBar:YES resetContent:NO];
     } else {
         [contentView addSubview:_collectionView];
         [_searchController setActive:NO];
+        if (@available(iOS 11.0, *))
+            [self setSearchBar:NO resetContent:NO];
     }
     self.view = contentView;
 }
@@ -242,6 +248,9 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     self.edgesForExtendedLayout = UIRectEdgeNone;
 
     [self setupSearchController];
+    UIButton *rendererButton = [VLCRendererDiscovererManager.sharedInstance setupRendererButton];
+    [rendererButton sizeToFit];
+    _rendererBarButton = [[UIBarButtonItem alloc] initWithCustomView:rendererButton];
 }
 
 - (void)setupSearchController
@@ -251,6 +260,10 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     _searchController.dimsBackgroundDuringPresentation = NO;
     _searchController.delegate = self;
     [self setSearchBar:YES resetContent:YES];
+    if (!self.usingTableViewToShowData) {
+        if (@available(iOS 11.0, *))
+            [self setSearchBar:NO resetContent:NO];
+    }
 }
 
 - (void)viewWillAppear:(BOOL)animated
@@ -259,6 +272,13 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     [self.collectionView.collectionViewLayout invalidateLayout];
     [self setViewFromDeviceOrientation];
     [self updateViewsForCurrentDisplayMode];
+
+    VLCRendererDiscovererManager *manager = VLCRendererDiscovererManager.sharedInstance;
+    if (manager.discoverers || manager.discoverers.count) {
+        // No discoverers has yet been started
+        [manager start];
+    }
+    manager.presentingViewController = self;
 }
 
 - (void)viewDidAppear:(BOOL)animated
@@ -345,6 +365,11 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 - (void)createSpotlightItem:(nonnull NSManagedObject *)mediaObject
 {
     if (![VLCKeychainCoordinator passcodeLockEnabled]) {
+        NSURL *uriRepresentation = mediaObject.objectID.URIRepresentation;
+        if (uriRepresentation == nil) {
+            return;
+        }
+
         self.userActivity = [[NSUserActivity alloc] initWithActivityType:kVLCUserActivityPlaying];
 
         MLFile *file = nil;
@@ -355,13 +380,12 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
         } else if ([mediaObject isKindOfClass:[MLFile class]]){
             file = (MLFile *)mediaObject;
         }
+
         self.userActivity.title = file.title;
         self.userActivity.contentAttributeSet = file.coreSpotlightAttributeSet;
-        self.userActivity.userInfo = @{@"playingmedia":mediaObject.objectID.URIRepresentation};
-
+        self.userActivity.userInfo = @{ @"playingmedia": uriRepresentation};
         self.userActivity.eligibleForSearch = YES;
         self.userActivity.eligibleForHandoff = YES;
-        //self.userActivity.contentUserAction = NSUserActivityContentUserActionPlay;
         [self.userActivity becomeCurrent];
     }
 }
@@ -390,12 +414,10 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
         [self.view addSubview:self.emptyLibraryView];
         self.navigationItem.rightBarButtonItems = nil;
     } else {
+        self.navigationItem.rightBarButtonItems = @[self.editButtonItem, _rendererBarButton];
         if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
-            UIBarButtonItem *toggleDisplayedView = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"tableViewIcon"] style:UIBarButtonItemStylePlain target:self action:@selector(toggleDisplayedView:)];
-            self.navigationItem.rightBarButtonItems = @[toggleDisplayedView, self.editButtonItem];
-            self.displayModeBarButtonItem = toggleDisplayedView;
-        } else {
-            self.navigationItem.rightBarButtonItem = self.editButtonItem;
+            self.displayModeBarButtonItem = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"tableViewIcon"] style:UIBarButtonItemStylePlain target:self action:@selector(toggleDisplayedView:)];
+            self.navigationItem.rightBarButtonItems = @[self.displayModeBarButtonItem, self.editButtonItem, _rendererBarButton];
         }
     }
     if (self.usingTableViewToShowData) {
@@ -422,11 +444,29 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 
 - (void)setSearchBar:(BOOL)enable resetContent:(BOOL)resetContent
 {
-    _tableView.tableHeaderView = enable ? _searchController.searchBar : nil;
-    if (resetContent) {
-        CGPoint contentOffset = _tableView.contentOffset;
-        contentOffset.y += CGRectGetHeight(_tableView.tableHeaderView.frame);
-        _tableView.contentOffset = contentOffset;
+    if (@available(iOS 11.0, *)) {
+        _searchController.dimsBackgroundDuringPresentation = NO;
+        [_searchController.searchBar sizeToFit];
+
+        // search bar text field background color
+        UITextField *searchTextField = [_searchController.searchBar valueForKey:@"searchField"];
+        UIView *backgroundView = searchTextField.subviews.firstObject;
+        backgroundView.backgroundColor = UIColor.whiteColor;
+        backgroundView.layer.cornerRadius = 10;
+        backgroundView.clipsToBounds = YES;
+
+        _searchController.hidesNavigationBarDuringPresentation = NO;
+        _searchController.obscuresBackgroundDuringPresentation = NO;
+        self.navigationItem.hidesSearchBarWhenScrolling = YES;
+        self.navigationItem.searchController = enable ? _searchController : nil;
+        self.definesPresentationContext = YES;
+    } else {
+        _tableView.tableHeaderView = enable ? _searchController.searchBar : nil;
+        if (resetContent) {
+            CGPoint contentOffset = _tableView.contentOffset;
+            contentOffset.y += CGRectGetHeight(_tableView.tableHeaderView.frame);
+            _tableView.contentOffset = contentOffset;
+        }
     }
 }
 
@@ -1001,13 +1041,12 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
 
     _isSelected = NO;
 
-    UIBarButtonItem *editButton = self.editButtonItem;
-    editButton.tintColor = [UIColor whiteColor];
-
-    if (!editing && self.navigationItem.rightBarButtonItems.lastObject == _selectAllBarButtonItem)
-        [self.navigationItem setRightBarButtonItems: [self.navigationItem.rightBarButtonItems subarrayWithRange:NSMakeRange(0, self.navigationItem.rightBarButtonItems.count - 1)]];
-    else
-        [self.navigationItem setRightBarButtonItems:editing ? [self.navigationItem.rightBarButtonItems arrayByAddingObject:_selectAllBarButtonItem] : [self.navigationItem rightBarButtonItems] animated:YES];
+    if (!editing) {
+        BOOL isIPad = UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad;
+        self.navigationItem.rightBarButtonItems = isIPad ?@[self.displayModeBarButtonItem, self.editButtonItem, _rendererBarButton] : @[self.editButtonItem, _rendererBarButton];
+    } else {
+        self.navigationItem.rightBarButtonItems = @[self.editButtonItem, _selectAllBarButtonItem];
+    }
 
     [self setSearchBar:!editing resetContent:!editing];
     self.tableView.allowsMultipleSelectionDuringEditing = editing;
@@ -1042,16 +1081,8 @@ static NSString *kUsingTableViewToShowData = @"UsingTableViewToShowData";
     [self.navigationController setToolbarHidden:!editing animated:YES];
 
     [UIView performWithoutAnimation:^{
-        [editButton setTitle:editing ? NSLocalizedString(@"BUTTON_CANCEL", nil) : NSLocalizedString(@"BUTTON_EDIT", nil)];
+        [self.editButtonItem setTitle:editing ? NSLocalizedString(@"BUTTON_CANCEL", nil) : NSLocalizedString(@"BUTTON_EDIT", nil)];
     }];
-
-    if (UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad) {
-        NSMutableArray *rightBarButtonItems = [self.navigationItem.rightBarButtonItems mutableCopy];
-        UIBarButtonItem *toggleDisplayedView = rightBarButtonItems[0];
-        toggleDisplayedView.enabled = !editing;
-        rightBarButtonItems[0] = toggleDisplayedView;
-        self.navigationItem.rightBarButtonItems = rightBarButtonItems;
-    }
 }
 
 - (void)toggleDisplayedView:(UIBarButtonItem *)button
