@@ -21,7 +21,6 @@
  *****************************************************************************/
 
 #import "VLCMovieViewController.h"
-#import "VLCExternalDisplayController.h"
 #import "VLCEqualizerView.h"
 #import "VLCMultiSelectionMenuView.h"
 #import "VLCPlaybackController.h"
@@ -36,12 +35,13 @@
 #import "VLCTrackSelectorView.h"
 #import "VLCMetadata.h"
 #import "UIDevice+VLC.h"
+#import "VLC_iOS-Swift.h"
 
 #define FORWARD_SWIPE_DURATION 30
 #define BACKWARD_SWIPE_DURATION 10
 #define SHORT_JUMP_DURATION 10
 
-#define ZOOM_SENSITIVITY 2.99f
+#define ZOOM_SENSITIVITY 5.f
 #define DEFAULT_FOV 80.f
 #define MAX_FOV 150.f
 #define MIN_FOV 20.f
@@ -54,7 +54,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
   VLCPanTypeProjection
 };
 
-@interface VLCMovieViewController () <UIGestureRecognizerDelegate, VLCMultiSelectionViewDelegate, VLCEqualizerViewUIDelegate>
+@interface VLCMovieViewController () <UIGestureRecognizerDelegate, VLCMultiSelectionViewDelegate, VLCEqualizerViewUIDelegate, VLCPlaybackControllerDelegate, VLCDeviceMotionDelegate, VLCRendererDiscovererManagerDelegate>
 {
     BOOL _controlsHidden;
     BOOL _videoFiltersHidden;
@@ -78,7 +78,6 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     BOOL _seekGestureEnabled;
     BOOL _closeGestureEnabled;
     BOOL _variableJumpDurationEnabled;
-    BOOL _mediaHasProjection;
     BOOL _playbackWillClose;
     BOOL _isTapSeeking;
     VLCMovieJumpState _previousJumpState;
@@ -111,13 +110,19 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     NSInteger _mediaDuration;
     NSInteger _numberOfTapSeek;
 
+    VLCDeviceMotion *_deviceMotion;
     CGFloat _fov;
     CGPoint _saveLocation;
-    CGSize _screenSizePixel;
+    CGSize _screenPixelSize;
+    UIInterfaceOrientation _lockedOrientation;
+
+    UIStackView *_navigationBarStackView;
+    UIButton *_rendererButton;
 }
 @property (nonatomic, strong) VLCMovieViewControlPanelView *controllerPanel;
 @property (nonatomic, strong) UIPopoverController *masterPopoverController;
-@property (nonatomic, strong) UIWindow *externalWindow;
+@property (nonatomic, strong) IBOutlet VLCPlayingExternallyView *playingExternalView;
+
 @end
 
 @implementation VLCMovieViewController
@@ -191,11 +196,6 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
                    name:VLCPlaybackControllerPlaybackDidStop
                  object:nil];
 
-    _playingExternallyTitle.text = NSLocalizedString(@"PLAYING_EXTERNALLY_TITLE", nil);
-    _playingExternallyDescription.text = NSLocalizedString(@"PLAYING_EXTERNALLY_DESC", nil);
-    if ([[UIDevice currentDevice] VLCHasExternalDisplay])
-        [self showOnExternalDisplay];
-
     self.trackNameLabel.text = self.artistNameLabel.text = self.albumNameLabel.text = @"";
 
     _movieView.userInteractionEnabled = NO;
@@ -236,12 +236,10 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [self setupControlPanel];
 
     CGRect screenBounds = [[UIScreen mainScreen] bounds];
-    CGFloat screenScale = [[UIScreen mainScreen] scale];
-    _screenSizePixel = CGSizeMake(screenBounds.size.width * screenScale, screenBounds.size.height * screenScale);
-    _saveLocation = CGPointMake(-1.f, -1.f);
+    _screenPixelSize = CGSizeMake(screenBounds.size.width, screenBounds.size.height);
 
     [self setupConstraints];
-
+    [self setupRendererDiscovererManager];
 }
 
 - (void)setupGestureRecognizers
@@ -360,7 +358,6 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
 - (void)setupNavigationbar
 {
-    //Needs to be a UIButton since we need it to work with constraints
     _doneButton = [[UIButton alloc] initWithFrame:CGRectZero];
     [_doneButton addTarget:self action:@selector(closePlayback:) forControlEvents:UIControlEventTouchUpInside];
     [_doneButton setTitle:NSLocalizedString(@"BUTTON_DONE", nil) forState:UIControlStateNormal];
@@ -370,21 +367,34 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     self.timeNavigationTitleView = [[[NSBundle mainBundle] loadNibNamed:@"VLCTimeNavigationTitleView" owner:self options:nil] objectAtIndex:0];
     self.timeNavigationTitleView.translatesAutoresizingMaskIntoConstraints = NO;
 
-    [self.navigationController.navigationBar addSubview:self.timeNavigationTitleView];
-    [self.navigationController.navigationBar addSubview:_doneButton];
+    if (_vpc.renderer != nil) {
+        [_rendererButton setSelected:YES];
+    }
+
+    _navigationBarStackView = [[UIStackView alloc] init];
+    _navigationBarStackView.translatesAutoresizingMaskIntoConstraints = NO;
+    _navigationBarStackView.spacing = 8;
+    _navigationBarStackView.axis = UILayoutConstraintAxisHorizontal;
+    _navigationBarStackView.alignment = UIStackViewAlignmentCenter;
+    [_navigationBarStackView addArrangedSubview:_doneButton];
+    [_navigationBarStackView addArrangedSubview:_timeNavigationTitleView];
+    [_navigationBarStackView addArrangedSubview:_rendererButton];
+
+    [self.navigationController.navigationBar addSubview:_navigationBarStackView];
 
     NSObject *guide = self.navigationController.navigationBar;
     if (@available(iOS 11.0, *)) {
         guide = self.navigationController.navigationBar.safeAreaLayoutGuide;
     }
-    [self.navigationController.view addConstraints: @[
-                                                      [NSLayoutConstraint constraintWithItem:_doneButton attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeLeft multiplier:1 constant:8],
-                                                      [NSLayoutConstraint constraintWithItem:_doneButton attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeCenterY multiplier:1 constant:0],
-                                                      [NSLayoutConstraint constraintWithItem:self.timeNavigationTitleView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:_doneButton attribute:NSLayoutAttributeRight multiplier:1 constant:0],
-                                                      [NSLayoutConstraint constraintWithItem:self.timeNavigationTitleView attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeRight multiplier:1 constant:0],
-                                                      [NSLayoutConstraint constraintWithItem:self.timeNavigationTitleView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeTop multiplier:1 constant:0],
-                                                      [NSLayoutConstraint constraintWithItem:self.timeNavigationTitleView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeBottom multiplier:1 constant:0],
-                                                      ]];
+
+    [NSLayoutConstraint activateConstraints:@[
+                                              [NSLayoutConstraint constraintWithItem:_navigationBarStackView attribute:NSLayoutAttributeCenterY relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeCenterY multiplier:1 constant:0],
+                                              [NSLayoutConstraint constraintWithItem:_navigationBarStackView attribute:NSLayoutAttributeLeft relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeLeft multiplier:1 constant:8],
+                                              [NSLayoutConstraint constraintWithItem:_navigationBarStackView attribute:NSLayoutAttributeRight relatedBy:NSLayoutRelationEqual toItem:guide attribute:NSLayoutAttributeRight multiplier:1 constant:-8],
+                                              [NSLayoutConstraint constraintWithItem:_navigationBarStackView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeTop multiplier:1 constant:0],
+                                              [NSLayoutConstraint constraintWithItem:_navigationBarStackView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:self.navigationController.navigationBar attribute:NSLayoutAttributeBottom multiplier:1 constant:0],
+                                              [NSLayoutConstraint constraintWithItem:_timeNavigationTitleView attribute:NSLayoutAttributeHeight relatedBy:NSLayoutRelationEqual toItem:_navigationBarStackView attribute:NSLayoutAttributeHeight multiplier:1 constant:0]
+                                              ]];
 }
 
 - (void)resetVideoFiltersSliders
@@ -401,6 +411,7 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [super viewWillAppear:animated];
 
     _vpc.delegate = self;
+    _lockedOrientation = UIInterfaceOrientationPortrait;
     [_vpc recoverPlaybackState];
     self.navigationController.navigationBar.barStyle = UIBarStyleBlack;
     self.navigationController.navigationBar.tintColor = [UIColor whiteColor];
@@ -415,9 +426,14 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
     [self updateDefaults];
 
-    //Disabling video gestures, media not init in the player yet.
-    [self enableNormalVideoGestures:NO];
     [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateDefaults) name:NSUserDefaultsDidChangeNotification object:nil];
+
+    VLCRendererDiscovererManager *manager = [VLCRendererDiscovererManager sharedInstance];
+    manager.presentingViewController = self;
+    manager.delegate = self;
+    if (_vpc.renderer || [[UIDevice currentDevice] VLCHasExternalDisplay]) {
+         [self showOnDisplay:_playingExternalView.displayView];
+    }
 }
 
 - (void)viewDidAppear:(BOOL)animated
@@ -433,14 +449,8 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     _multiSelectionView.shuffleMode = _vpc.isShuffleMode;
 
     //Media is loaded in the media player, checking the projection type and configuring accordingly.
-    _fov = 0.f;
-    _mediaHasProjection = NO;
-    if ([_vpc currentMediaProjection] == VLCMediaProjectionEquiRectangular) {
-        _fov = DEFAULT_FOV;
-        _mediaHasProjection = YES;
-    }
-
-    [self enableNormalVideoGestures:!_mediaHasProjection];
+    [self setupForMediaProjection];
+    [VLCRendererDiscovererManager.sharedInstance start];
 }
 
 - (void)viewDidLayoutSubviews
@@ -512,6 +522,12 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     [[NSUserDefaults standardUserDefaults] setBool:_displayRemainingTime forKey:kVLCShowRemainingTime];
 }
 
+- (void)viewDidDisappear:(BOOL)animated
+{
+    [super viewDidDisappear:animated];
+    [self.deviceMotion stopDeviceMotion];
+}
+
 - (BOOL)canBecomeFirstResponder
 {
     return YES;
@@ -635,7 +651,8 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
                                                                      _multiSelectionView.chapterSelectorButton,
                                                                      _multiSelectionView.repeatButton,
                                                                      _multiSelectionView.shuffleButton,
-                                                                     _controllerPanel.volumeView]];
+                                                                     _controllerPanel.volumeView,
+                                                                     _rendererButton]];
 
     [[UIDevice currentDevice] isiPhoneX] ? [items addObject:_tapToToggleiPhoneXRatioRecognizer]
                                          : [items addObject:_tapToSeekRecognizer];
@@ -645,21 +662,23 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
 - (void)handlePinchGesture:(UIPinchGestureRecognizer *)recognizer
 {
-    if (!_closeGestureEnabled || isnan(recognizer.velocity))
-        return;
+    CGFloat diff = DEFAULT_FOV * -(ZOOM_SENSITIVITY * recognizer.velocity / _screenPixelSize.width);
 
-    CGFloat diff = DEFAULT_FOV * -(ZOOM_SENSITIVITY * recognizer.velocity / _screenSizePixel.width);
-
-    if (_mediaHasProjection) {
-        if ([_vpc updateViewpoint:0 pitch:0 roll:0 fov:diff absolute:NO]) {
-            //Checking for fov value in case of
-            _fov = MAX(MIN(_fov + diff, MAX_FOV), MIN_FOV);
-        }
-    } else if (recognizer.velocity < 0.) {
+    if ([_vpc currentMediaIs360Video]) {
+        [self zoom360Video:diff];
+    } else if (recognizer.velocity < 0. && _closeGestureEnabled) {
         [self minimizePlayback:nil];
     }
 }
 
+- (void)zoom360Video:(CGFloat)zoom
+{
+    if ([_vpc updateViewpoint:0 pitch:0 roll:0 fov:zoom absolute:NO]) {
+        //clamp Fov between min and max fov
+        _fov = MAX(MIN(_fov + zoom, MAX_FOV), MIN_FOV);
+    }
+}
+
 - (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
 {
     if (touch.view != self.view)
@@ -838,15 +857,37 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
     return [super nextResponder];
 }
 
-- (void)enableNormalVideoGestures:(BOOL)enable
+- (VLCDeviceMotion *)deviceMotion
 {
-    [_tapRecognizer setEnabled:enable];
-    [_swipeRecognizerUp setEnabled:enable];
-    [_swipeRecognizerDown setEnabled:enable];
-    [_swipeRecognizerLeft setEnabled:enable];
-    [_swipeRecognizerRight setEnabled:enable];
+    if (!_deviceMotion) {
+        _deviceMotion = [VLCDeviceMotion new];
+        _deviceMotion.delegate = self;
+    }
+    return _deviceMotion;
 }
 
+- (void)setupForMediaProjection
+{
+    BOOL mediaHasProjection = [_vpc currentMediaIs360Video];
+    _fov = mediaHasProjection ? DEFAULT_FOV : 0.f;
+
+    [_panRecognizer setEnabled:mediaHasProjection];
+    [_swipeRecognizerUp setEnabled:!mediaHasProjection];
+    [_swipeRecognizerDown setEnabled:!mediaHasProjection];
+    [_swipeRecognizerLeft setEnabled:!mediaHasProjection];
+    [_swipeRecognizerRight setEnabled:!mediaHasProjection];
+
+    if (mediaHasProjection) {
+        [self.deviceMotion startDeviceMotion];
+    }
+}
+
+- (void)deviceMotionHasAttitudeWithDeviceMotion:(VLCDeviceMotion *)deviceMotion pitch:(double)pitch yaw:(double)yaw roll:(double)roll
+{
+    if (_panRecognizer.state != UIGestureRecognizerStateChanged || UIGestureRecognizerStateBegan) {
+        [_vpc updateViewpoint:yaw pitch:pitch roll:roll fov:_fov absolute:YES];
+    }
+}
 #pragma mark - controls
 
 - (IBAction)closePlayback:(id)sender
@@ -974,11 +1015,11 @@ typedef NS_ENUM(NSInteger, VLCPanType) {
 
     float audioDelay = controller.audioDelay;
     self.audioDelaySlider.value = audioDelay;
-    self.audioDelayIndicator.text = [NSString stringWithFormat:@"%1.2f s", audioDelay];
+    self.audioDelayIndicator.text = [NSString stringWithFormat:@"%d ms", (int) audioDelay];
 
     float subtitleDelay = controller.subtitleDelay;
     self.spuDelaySlider.value = subtitleDelay;
-    self.spuDelayIndicator.text = [NSString stringWithFormat:@"%1.00f s", subtitleDelay];
+    self.spuDelayIndicator.text = [NSString stringWithFormat:@"%d ms", (int) subtitleDelay];
 
     [self _resetIdleTimer];
 }
@@ -1178,6 +1219,9 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 - (void)toggleUILock
 {
     _interfaceIsLocked = !_interfaceIsLocked;
+    if (_interfaceIsLocked) {
+        _lockedOrientation = [[UIApplication sharedApplication] statusBarOrientation];
+    }
 
     NSArray *items = [self itemsForInterfaceLock];
 
@@ -1289,18 +1333,6 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
     }
 }
 
-- (void)toggleiPhoneXAspectRatio
-{
-    if (@available(iOS 11.0, *)) {
-        BOOL isFullScreen = CGRectEqualToRect(_movieView.frame , self.view.frame);
-
-        CGRect frameWithoutNotch = self.view.safeAreaLayoutGuide.layoutFrame;
-        [UIView animateWithDuration:0.3 animations:^{
-            _movieView.frame = isFullScreen ? frameWithoutNotch : self.view.frame;
-        }];
-    }
-}
-
 - (BOOL)prefersHomeIndicatorAutoHidden
 {
     return YES;
@@ -1318,9 +1350,12 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
         panType = VLCPanTypeBrightness;
 
     // only check for seeking gesture if on iPad , will overwrite last statements if true
-    if ([deviceType isEqualToString:@"iPad"]) {
-        if (location.y < 110)
-            panType = VLCPanTypeSeek;
+    if ([deviceType isEqualToString:@"iPad"] && location.y < 110) {
+        panType = VLCPanTypeSeek;
+    }
+
+    if ([_vpc currentMediaIs360Video]) {
+        panType = VLCPanTypeProjection;
     }
 
     return panType;
@@ -1330,86 +1365,100 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 {
     CGFloat panDirectionX = [panRecognizer velocityInView:self.view].x;
     CGFloat panDirectionY = [panRecognizer velocityInView:self.view].y;
-
     if (panRecognizer.state == UIGestureRecognizerStateBegan) {
-        _currentPanType = _mediaHasProjection ? VLCPanTypeProjection : [self detectPanTypeForPan:panRecognizer];
+        _currentPanType = [self detectPanTypeForPan:panRecognizer];
+        if ([_vpc currentMediaIs360Video]) {
+            _saveLocation =  [panRecognizer locationInView:self.view];
+            [_deviceMotion stopDeviceMotion];
+        }
     }
+    
+    switch (_currentPanType) {
+        case VLCPanTypeSeek: {
+            if (!_seekGestureEnabled)
+                return;
+            double timeRemainingDouble = (-[_vpc remainingTime].intValue*0.001);
+            int timeRemaining = timeRemainingDouble;
 
-    if (_currentPanType == VLCPanTypeSeek) {
-        if (!_seekGestureEnabled)
-            return;
-        double timeRemainingDouble = (-[_vpc remainingTime].intValue*0.001);
-        int timeRemaining = timeRemainingDouble;
+            if (panDirectionX > 0) {
+                if (timeRemaining > 2 ) // to not go outside duration , video will stop
+                    [_vpc jumpForward:1];
+            } else
+                [_vpc jumpBackward:1];
 
-        if (panDirectionX > 0) {
-            if (timeRemaining > 2 ) // to not go outside duration , video will stop
-                [_vpc jumpForward:1];
-        } else
-            [_vpc jumpBackward:1];
-    } else if (_currentPanType == VLCPanTypeVolume) {
-        if (!_volumeGestureEnabled)
-            return;
-        MPMusicPlayerController *musicPlayer = [MPMusicPlayerController applicationMusicPlayer];
+            break;
+        case VLCPanTypeVolume:
+
+            if (!_volumeGestureEnabled)
+                return;
+            MPMusicPlayerController *musicPlayer = [MPMusicPlayerController applicationMusicPlayer];
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated"
-        // there is no replacement for .volume which we want to use since Apple's susggestion is to not use their overlays
-        // but switch to the volume slider exclusively. meh.
-        if (panDirectionY > 0)
-            musicPlayer.volume -= 0.01;
-        else
-            musicPlayer.volume += 0.01;
+            // there is no replacement for .volume which we want to use since Apple's susggestion is to not use their overlays
+            // but switch to the volume slider exclusively. meh.
+            if (panDirectionY > 0)
+                musicPlayer.volume -= 0.01;
+            else
+                musicPlayer.volume += 0.01;
 #pragma clang diagnostic pop
-    } else if (_currentPanType == VLCPanTypeBrightness) {
-        if (!_brightnessGestureEnabled)
-            return;
-        CGFloat brightness = [UIScreen mainScreen].brightness;
+        } break;
+        case VLCPanTypeBrightness: {
+            if (!_brightnessGestureEnabled)
+                return;
+            CGFloat brightness = [UIScreen mainScreen].brightness;
 
-        if (panDirectionY > 0)
-            brightness = brightness - 0.01;
-        else
-            brightness = brightness + 0.01;
+            if (panDirectionY > 0)
+                brightness = brightness - 0.01;
+            else
+                brightness = brightness + 0.01;
 
-        // Sanity check since -[UIScreen brightness] does not go by 0.01 steps
-        if (brightness > 1.0)
-            brightness = 1.0;
-        else if (brightness < 0.0)
-            brightness = 0.0;
+            // Sanity check since -[UIScreen brightness] does not go by 0.01 steps
+            if (brightness > 1.0)
+                brightness = 1.0;
+            else if (brightness < 0.0)
+                brightness = 0.0;
 
-        NSAssert(brightness >= 0 && brightness <= 1, @"Brightness must be within 0 and 1 (it is %f)", brightness);
+            NSAssert(brightness >= 0 && brightness <= 1, @"Brightness must be within 0 and 1 (it is %f)", brightness);
 
-        [[UIScreen mainScreen] setBrightness:brightness];
+            [[UIScreen mainScreen] setBrightness:brightness];
 
-        NSString *brightnessHUD = [NSString stringWithFormat:@"%@: %@ %%", NSLocalizedString(@"VFILTER_BRIGHTNESS", nil), [[[NSString stringWithFormat:@"%f",(brightness*100)] componentsSeparatedByString:@"."] objectAtIndex:0]];
-        [self.statusLabel showStatusMessage:brightnessHUD];
-    } else if (_currentPanType == VLCPanTypeProjection) {
-
-        CGPoint tmp = [panRecognizer locationInView:self.view];
-        CGFloat changeX = 0.f;
-        CGFloat changeY = 0.f;
-
-        if (_saveLocation.x != -1.f && _saveLocation.y != -1.f) {
-            changeX = tmp.x - _saveLocation.x;
-            changeY = tmp.y - _saveLocation.y;
-        }
-
-        _saveLocation = [panRecognizer locationInView:self.view];
-
-        //screenSizePixel width is used twice to get a constant speed on the movement.
-        CGFloat yaw = _fov * -changeX / _screenSizePixel.width;
-        CGFloat pitch = _fov * -changeY / _screenSizePixel.width;
-
-        [_vpc updateViewpoint:yaw pitch:pitch roll:0 fov:0 absolute:NO];
+            NSString *brightnessHUD = [NSString stringWithFormat:@"%@: %@ %%", NSLocalizedString(@"VFILTER_BRIGHTNESS", nil), [[[NSString stringWithFormat:@"%f",(brightness*100)] componentsSeparatedByString:@"."] objectAtIndex:0]];
+            [self.statusLabel showStatusMessage:brightnessHUD];
+        } break;
+        case VLCPanTypeProjection: {
+            [self updateProjectionWithPanRecognizer:panRecognizer];
+        } break;
+        case VLCPanTypeNone: {
+        } break;
     }
 
     if (panRecognizer.state == UIGestureRecognizerStateEnded) {
         _currentPanType = VLCPanTypeNone;
-
-        //Invalidate saved location when the gesture is ended
-        if (_mediaHasProjection)
-            _saveLocation = CGPointMake(-1.f, -1.f);
+        if ([_vpc currentMediaIs360Video]) {
+            [_deviceMotion lastAngleWithYaw:_vpc.yaw pitch:_vpc.pitch roll:_vpc.roll];
+            [_deviceMotion startDeviceMotion];
+        }
     }
 }
 
+- (void)updateProjectionWithPanRecognizer:(UIPanGestureRecognizer *)panGestureRecognizer
+{
+    CGPoint newLocationInView = [panGestureRecognizer locationInView:self.view];
+    
+    CGFloat diffX = newLocationInView.x - _saveLocation.x;
+    CGFloat diffY = newLocationInView.y - _saveLocation.y;
+    _saveLocation = newLocationInView;
+
+    //screenSizePixel width is used twice to get a constant speed on the movement.
+    CGFloat yaw = _fov * -diffX / _screenPixelSize.width;
+    CGFloat pitch = _fov * -diffY / _screenPixelSize.width;
+
+    CGFloat newYaw = _vpc.yaw + yaw;
+    CGFloat newPitch = _vpc.pitch + pitch;
+
+    [_vpc updateViewpoint:newYaw pitch:newPitch roll:_vpc.roll fov:_vpc.fov absolute:YES];
+}
+
 - (void)swipeRecognized:(UISwipeGestureRecognizer*)swipeRecognizer
 {
     if (!_seekGestureEnabled)
@@ -1538,13 +1587,15 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
         _vpc.playbackRate = speed;
         self.playbackSpeedIndicator.text = [NSString stringWithFormat:@"%.2fx", speed];
     } else if (sender == _audioDelaySlider) {
-        double delay = sender.value;
+        int delay = ((int) round(sender.value / 50.)) * 50;
         _vpc.audioDelay = delay;
-        _audioDelayIndicator.text = [NSString stringWithFormat:@"%1.2f s", delay];
+        [sender setValue:delay animated:NO];
+        _audioDelayIndicator.text = [NSString stringWithFormat:@"%d ms", delay];
     } else if (sender == _spuDelaySlider) {
-        double delay = sender.value;
+        int delay = (int) (round(sender.value / 50.)) * 50;
         _vpc.subtitleDelay = delay;
-        _spuDelayIndicator.text = [NSString stringWithFormat:@"%1.00f s", delay];
+        [sender setValue:delay animated:NO];
+        _spuDelayIndicator.text = [NSString stringWithFormat:@"%d ms", delay];
     }
 
     [self _resetIdleTimer];
@@ -1571,9 +1622,11 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 
 #pragma mark - autorotation
 
-- (BOOL)rotationIsDisabled
+- (UIInterfaceOrientationMask)supportedInterfaceOrientations
 {
-    return _interfaceIsLocked;
+    BOOL orientationIslocked = _interfaceIsLocked || [_vpc currentMediaIs360Video];
+    UIInterfaceOrientationMask maskFromOrientation = 1 << _lockedOrientation;
+    return orientationIslocked ? maskFromOrientation :  UIInterfaceOrientationMaskAll;
 }
 
 - (BOOL)shouldAutorotate
@@ -1589,7 +1642,6 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
         [self setControlsHidden:NO animated:YES];
 }
 
-// >= iOS 8
 - (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
 {
     [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
@@ -1614,43 +1666,45 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 
 #pragma mark - External Display
 
-- (void)showOnExternalDisplay
+- (void)showOnDisplay:(UIView *)view
 {
-    UIScreen *screen = [UIScreen screens][1];
-    screen.overscanCompensation = UIScreenOverscanCompensationInsetApplicationFrame;
-
-    self.externalWindow = [[UIWindow alloc] initWithFrame:screen.bounds];
-
-    UIViewController *controller = [[VLCExternalDisplayController alloc] init];
-    self.externalWindow.rootViewController = controller;
-    [controller.view addSubview:_movieView];
-    controller.view.frame = screen.bounds;
-    _movieView.frame = screen.bounds;
-
-    self.playingExternallyView.hidden = NO;
-    self.externalWindow.screen = screen;
-    self.externalWindow.hidden = NO;
-}
-
-- (void)hideFromExternalDisplay
-{
-    [self.view addSubview:_movieView];
-    [self.view sendSubviewToBack:_movieView];
-    _movieView.frame = self.view.frame;
-
-    self.playingExternallyView.hidden = YES;
-    self.externalWindow.hidden = YES;
-    self.externalWindow = nil;
+    // if we don't have a renderer we're mirroring and don't want to show the dialog
+    BOOL displayExternally = _vpc.renderer && view != _movieView;
+    [_playingExternalView shouldDisplay:displayExternally];
+    [_playingExternalView updateUIWithRendererItem:_vpc.renderer];
+    _vpc.videoOutputView = view;
+    _artworkImageView.hidden = displayExternally;
 }
 
 - (void)handleExternalScreenDidConnect:(NSNotification *)notification
 {
-    [self showOnExternalDisplay];
+    [self showOnDisplay:_playingExternalView.displayView];
 }
 
 - (void)handleExternalScreenDidDisconnect:(NSNotification *)notification
 {
-    [self hideFromExternalDisplay];
+    [self showOnDisplay:_movieView];
 }
 
+#pragma mark - Renderers
+
+- (void)setupRendererDiscovererManager
+{
+    // Create a renderer button for VLCMovieViewController
+    _rendererButton = [VLCRendererDiscovererManager.sharedInstance setupRendererButton];
+    [VLCRendererDiscovererManager.sharedInstance addSelectionHandler:^(VLCRendererItem * item) {
+        if (item) {
+            [self showOnDisplay:_playingExternalView.displayView];
+        } else {
+            [self removedCurrentRendererItem:_vpc.renderer];
+        }
+    }];
+}
+
+#pragma mark - VLCRendererDiscovererManagerDelegate
+
+- (void)removedCurrentRendererItem:(VLCRendererItem *)item
+{
+    [self showOnDisplay:_movieView];
+}
 @end
