@@ -2,7 +2,7 @@
  * VLCOpenNetworkStreamViewController.m
  * VLC for iOS
  *****************************************************************************
- * Copyright (c) 2013-2015 VideoLAN. All rights reserved.
+ * Copyright (c) 2013-2018 VideoLAN. All rights reserved.
  * $Id$
  *
  * Authors: Felix Paul Kühne <fkuehne # videolan.org>
@@ -19,7 +19,7 @@
 #import "VLCStreamingHistoryCell.h"
 #import "UIDevice+VLC.h"
 
-@interface VLCOpenNetworkStreamViewController () <UITableViewDataSource, UITableViewDelegate, UITextFieldDelegate, UIAlertViewDelegate, VLCStreamingHistoryCellMenuItemProtocol>
+@interface VLCOpenNetworkStreamViewController () <UITableViewDataSource, UITableViewDelegate, UITextFieldDelegate, VLCStreamingHistoryCellMenuItemProtocol>
 {
     NSMutableArray *_recentURLs;
     NSMutableDictionary *_recentURLTitles;
@@ -178,22 +178,33 @@
 
 - (IBAction)openButtonAction:(id)sender
 {
-    if ([self.urlField.text length] > 0) {
-        if (!self.privateToggleSwitch.on) {
-            NSString *urlString = self.urlField.text;
-            if ([_recentURLs indexOfObject:urlString] != NSNotFound)
-                [_recentURLs removeObject:urlString];
+    if ([self.urlField.text length] <= 0 || [NSURL URLWithString:self.urlField.text] == nil) {
+        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:NSLocalizedString(@"URL_NOT_SUPPORTED", nil)
+                                                                                 message:nil
+                                                                          preferredStyle:UIAlertControllerStyleAlert];
 
-            if (_recentURLs.count >= 100)
-                [_recentURLs removeLastObject];
-            [_recentURLs addObject:urlString];
-            [[NSUbiquitousKeyValueStore defaultStore] setArray:_recentURLs forKey:kVLCRecentURLs];
+        UIAlertAction *okAction = [UIAlertAction actionWithTitle:NSLocalizedString(@"BUTTON_OK", nil)
+                                                           style:UIAlertActionStyleCancel
+                                                         handler:nil];
 
-            [self.historyTableView reloadData];
-        }
-        [self.urlField resignFirstResponder];
-        [self _openURLStringAndDismiss:self.urlField.text];
+        [alertController addAction:okAction];
+        [self presentViewController:alertController animated:YES completion:nil];
+        return;
     }
+    if (!self.privateToggleSwitch.on) {
+        NSString *urlString = self.urlField.text;
+        if ([_recentURLs indexOfObject:urlString] != NSNotFound)
+            [_recentURLs removeObject:urlString];
+
+        if (_recentURLs.count >= 100)
+            [_recentURLs removeLastObject];
+        [_recentURLs addObject:urlString];
+        [[NSUbiquitousKeyValueStore defaultStore] setArray:_recentURLs forKey:kVLCRecentURLs];
+
+        [self.historyTableView reloadData];
+    }
+    [self.urlField resignFirstResponder];
+    [self _openURLStringAndDismiss:self.urlField.text];
 }
 
 - (void)renameStreamFromCell:(UITableViewCell *)cell {
@@ -201,50 +212,37 @@
     NSString *renameString = NSLocalizedString(@"BUTTON_RENAME", nil);
     NSString *cancelString = NSLocalizedString(@"BUTTON_CANCEL", nil);
 
-    if (@available(iOS 8, *))
-    {
-        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:renameString
-                                                                                 message:nil
-                                                                          preferredStyle:UIAlertControllerStyleAlert];
+    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:renameString
+                                                                             message:nil
+                                                                      preferredStyle:UIAlertControllerStyleAlert];
 
-        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelString
-                                                        style:UIAlertActionStyleCancel
-                                                             handler:nil];
-        UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
-            NSString *streamTitle = alertController.textFields.firstObject.text;
-            [self renameStreamWithTitle:streamTitle atIndex:cellIndexPath.row];
-        }];
+    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:cancelString
+                                                           style:UIAlertActionStyleCancel
+                                                         handler:nil];
+    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
+        NSString *streamTitle = alertController.textFields.firstObject.text;
+        [self renameStreamWithTitle:streamTitle atIndex:cellIndexPath.row];
+    }];
 
-        [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
-            textField.text = cell.textLabel.text;
+    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
+        textField.text = cell.textLabel.text;
 
-            [[NSNotificationCenter defaultCenter] addObserverForName:UITextFieldTextDidChangeNotification
-                                                              object:textField
-                                                               queue:[NSOperationQueue mainQueue]
-                                                          usingBlock:^(NSNotification * _Nonnull note) {
-                okAction.enabled = (textField.text.length != 0);
-            }];
-        }];
+        [[NSNotificationCenter defaultCenter] addObserverForName:UITextFieldTextDidChangeNotification
+                                                          object:textField
+                                                           queue:[NSOperationQueue mainQueue]
+                                                      usingBlock:^(NSNotification * _Nonnull note) {
+                                                          okAction.enabled = (textField.text.length != 0);
+                                                      }];
+    }];
 
-        [alertController addAction:cancelAction];
-        [alertController addAction:okAction];
+    [alertController addAction:cancelAction];
+    [alertController addAction:okAction];
 
-        [self presentViewController:alertController animated:YES completion:nil];
-    }
-    else
-    {
-        UIAlertView *alertView = [[UIAlertView alloc] init];
-        alertView.delegate = self;
-        alertView.alertViewStyle = UIAlertViewStylePlainTextInput;
-        alertView.title = renameString;
-        alertView.tag = cellIndexPath.row; // Dirty...
-        [alertView addButtonWithTitle:cancelString];
-        [alertView addButtonWithTitle:@"OK"];
-        [alertView show];
-    }
+    [self presentViewController:alertController animated:YES completion:nil];
 }
 
-- (void)renameStreamWithTitle:(NSString *)title atIndex:(NSInteger)index {
+- (void)renameStreamWithTitle:(NSString *)title atIndex:(NSInteger)index
+{
     [_recentURLTitles setObject:title forKey:[@(index) stringValue]];
     [[NSUbiquitousKeyValueStore defaultStore] setDictionary:_recentURLTitles forKey:kVLCRecentURLTitles];
     [[NSOperationQueue mainQueue] addOperationWithBlock:^{
@@ -252,13 +250,6 @@
     }];
 }
 
-#pragma mark - alert view delegate (iOS 7)
-
-- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
-    NSString *streamTitle = [alertView textFieldAtIndex:0].text;
-    [self renameStreamWithTitle:streamTitle atIndex:alertView.tag]; // Dirty...
-}
-
 #pragma mark - table view data source
 - (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
 {
@@ -379,7 +370,8 @@ forRowAtIndexPath:(NSIndexPath *)indexPath
         request.HTTPMethod = @"HEAD";
 
         NSURLResponse *response = nil;
-        [NSURLConnection sendSynchronousRequest:request returningResponse:&response error:nil];
+        NSError *error = nil;
+        [self sendSynchronousRequest:request returningResponse:&response error:&error];
         NSInteger httpStatus = [(NSHTTPURLResponse *)response statusCode];
 
         if (httpStatus == 200) {
@@ -419,6 +411,29 @@ forRowAtIndexPath:(NSIndexPath *)indexPath
     return fileSubtitlePath;
 }
 
+- (NSData *)sendSynchronousRequest:(NSURLRequest *)request returningResponse:(NSURLResponse **)response error:(NSError **)error
+{
+    NSError __block *erreur = NULL;
+    NSData __block *data;
+    BOOL __block reqProcessed = false;
+    NSURLResponse __block *urlResponse;
+
+    [[[NSURLSession sharedSession] dataTaskWithRequest:request completionHandler:^(NSData * _Nullable _data, NSURLResponse * _Nullable _response, NSError * _Nullable _error) {
+        urlResponse = _response;
+        erreur = _error;
+        data = _data;
+        reqProcessed = true;
+    }] resume];
+
+    while (!reqProcessed) {
+        [NSThread sleepForTimeInterval:0];
+    }
+
+    *response = urlResponse;
+    *error = erreur;
+    return data;
+}
+
 #pragma mark - text view delegate
 - (BOOL)textFieldShouldReturn:(UITextField *)textField
 {
