@@ -22,6 +22,7 @@ extern NSString *const VLCPlaybackControllerPlaybackMetadataDidChange;
 extern NSString *const VLCPlaybackControllerPlaybackPositionUpdated;
 
 @class VLCPlaybackController;
+@class VLCMetaData;
 
 @protocol VLCPlaybackControllerDelegate <NSObject>
 @optional
@@ -33,64 +34,93 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
           forPlaybackController:(VLCPlaybackController *)controller;
 - (void)prepareForMediaPlayback:(VLCPlaybackController *)controller;
 - (void)showStatusMessage:(NSString *)statusMessage forPlaybackController:(VLCPlaybackController *)controller;
-- (void)displayMetadataForPlaybackController:(VLCPlaybackController *)controller
-                                       title:(NSString *)title
-                                     artwork:(UIImage *)artwork
-                                      artist:(NSString *)artist
-                                       album:(NSString *)album
-                                   audioOnly:(BOOL)audioOnly;
+- (void)displayMetadataForPlaybackController:(VLCPlaybackController *)controller metadata:(VLCMetaData *)metadata;
 
 @end
 
 @interface VLCPlaybackController : NSObject <VLCEqualizerViewDelegate>
 
-@property (nonatomic, readonly) VLCMediaListPlayer *listPlayer;
-@property (nonatomic, readonly) VLCMediaPlayer *mediaPlayer;
-
 @property (nonatomic, strong) UIView *videoOutputView;
 
-@property (nonatomic, readwrite) BOOL sessionWillRestart;
-
-@property (nonatomic, strong) NSURL *url;
 @property (nonatomic, strong) NSURL *successCallback;
 @property (nonatomic, strong) NSURL *errorCallback;
 
-@property (nonatomic, strong) NSString *pathToExternalSubtitlesFile;
 @property (nonatomic, retain) VLCMediaList *mediaList;
-@property (nonatomic, readwrite) int itemInMediaListToBePlayedFirst;
 
-#if TARGET_OS_IOS
-/* returns nil if currently playing item is not a MLFile, e.g. a url */
-@property (nonatomic, strong, readonly) MLFile *currentlyPlayingMediaFile;
-#endif
+/* returns nil if currently playing item is not available,*/
+
+@property (nonatomic, strong, readonly) VLCMedia *currentlyPlayingMedia;
 
 @property (nonatomic, weak) id<VLCPlaybackControllerDelegate> delegate;
 
 @property (nonatomic, readonly) VLCMediaPlayerState mediaPlayerState;
+@property (nonatomic, readonly) VLCMetaData *metadata;
+
 @property (nonatomic, readonly) NSInteger mediaDuration;
 @property (nonatomic, readonly) BOOL isPlaying;
+@property (nonatomic, readonly) BOOL willPlay;
 @property (nonatomic, readwrite) VLCRepeatMode repeatMode;
 @property (nonatomic, assign, getter=isShuffleMode) BOOL shuffleMode;
 @property (nonatomic, readwrite) float playbackRate; // default = 1.0
-@property (nonatomic, readwrite) float audioDelay; // in seconds, default = 0.0
-@property (nonatomic, readwrite) float subtitleDelay; // in seconds, default = 0.0
-@property (nonatomic, readonly) BOOL currentMediaHasChapters;
-@property (nonatomic, readonly) BOOL currentMediaHasTrackToChooseFrom;
-@property (nonatomic, readonly) BOOL activePlaybackSession;
-@property (nonatomic, readonly) BOOL audioOnlyPlaybackSession;
-@property (nonatomic, readonly) NSString *mediaTitle;
-@property (nonatomic, readwrite) BOOL fullscreenSessionRequested;
+@property (nonatomic, readwrite) float audioDelay; // in milliseconds, default = 0.0
+@property (nonatomic, readwrite) float playbackPosition; // in seconds, default = 0.0
+@property (nonatomic, readwrite) float subtitleDelay; // in milliseconds, default = 0.0
+
+@property (nonatomic, readwrite) float hue; // default = 0.0
+@property (nonatomic, readwrite) float contrast; // default = 1.0
+@property (nonatomic, readwrite) float brightness; // default = 1.0
+@property (nonatomic, readwrite) float saturation; // default = 1.0
+@property (nonatomic, readwrite) float gamma; // default = 1.0
+
+@property (nonatomic, readonly) CGFloat yaw; //  between ]-180;180]
+@property (nonatomic, readonly) CGFloat pitch; // ]-90;90]
+@property (nonatomic, readonly) CGFloat roll; // ]-180;180]
+@property (nonatomic, readonly) CGFloat fov; // ]0;180[ (default 80.)
+
+@property (readonly) NSInteger indexOfCurrentAudioTrack;
+@property (readonly) NSInteger indexOfCurrentSubtitleTrack;
+@property (readonly) NSInteger indexOfCurrentTitle;
+@property (readonly) NSInteger indexOfCurrentChapter;
+@property (readonly) NSInteger numberOfAudioTracks;
+@property (readonly) NSInteger numberOfVideoSubtitlesIndexes;
+@property (readonly) NSInteger numberOfTitles;
+@property (readonly) NSInteger numberOfChaptersForCurrentTitle;
+@property (assign, readonly) BOOL currentMediaHasTrackToChooseFrom;
+@property (assign, readwrite) BOOL fullscreenSessionRequested;
+@property (assign, readonly) BOOL isSeekable;
+@property (assign, readonly) BOOL currentMediaIs360Video;
+@property (readonly) NSNumber *playbackTime;
 @property (nonatomic, readonly) NSDictionary *mediaOptionsDictionary;
-@property (nonatomic, readonly) NSTimer* sleepTimer;
+@property (nonatomic, readonly) NSTimer *sleepTimer;
+
+@property (nonatomic) VLCRendererItem *renderer;
 
 + (VLCPlaybackController *)sharedInstance;
-
+- (VLCTime *)playedTime;
+#pragma mark - playback
 - (void)startPlayback;
 - (void)stopPlayback;
-
 - (void)playPause;
-- (void)forward;
-- (void)backward;
+- (void)play;
+- (void)pause;
+- (void)next;
+- (void)previous;
+- (void)jumpForward:(int)interval;
+- (void)jumpBackward:(int)interval;
+- (void)toggleRepeatMode;
+- (void)resetFilters;
+- (VLCTime *)remainingTime;
+
+- (NSString *)audioTrackNameAtIndex:(NSInteger)index;
+- (NSString *)videoSubtitleNameAtIndex:(NSInteger)index;
+- (NSDictionary *)titleDescriptionsDictAtIndex:(NSInteger)index;
+- (NSDictionary *)chapterDescriptionsDictAtIndex:(NSInteger)index;
+- (void)selectAudioTrackAtIndex:(NSInteger)index;
+- (void)selectVideoSubtitleAtIndex:(NSInteger)index;
+- (void)selectTitleAtIndex:(NSInteger)index;
+- (void)selectChapterAtIndex:(NSInteger)index;
+- (void)setAudioPassthrough:(BOOL)shouldPass;
+
 - (void)switchAspectRatio;
 - (void)switchIPhoneXFullScreen;
 #if !TARGET_OS_TV
@@ -102,10 +132,8 @@ currentMediaHasTrackToChooseFrom:(BOOL)currentMediaHasTrackToChooseFrom
 
 - (void)setNeedsMetadataUpdate;
 - (void)scheduleSleepTimerWithInterval:(NSTimeInterval)timeInterval;
-
-- (void)playMediaList:(VLCMediaList *)mediaList firstIndex:(NSInteger)index;
-- (void)playURL:(NSURL *)url successCallback:(NSURL*)successCallback errorCallback:(NSURL *)errorCallback;
-- (void)playURL:(NSURL *)url subtitlesFilePath:(NSString *)subsFilePath;
-- (void)remoteControlReceivedWithEvent:(UIEvent *)event;
+- (void)performNavigationAction:(VLCMediaPlaybackNavigationAction)action;
+- (void)playMediaList:(VLCMediaList *)mediaList firstIndex:(NSInteger)index subtitlesFilePath:(NSString *)subsFilePath;
+- (void)openVideoSubTitlesFromFile:(NSString *)pathToFile;
 
 @end
