@@ -54,6 +54,7 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     NSTimer *_sleepTimer;
 
     NSUInteger _currentAspectRatio;
+    BOOL _isInFillToScreen;
 
     UIView *_videoOutputViewWrapper;
     UIView *_actualVideoOutputView;
@@ -238,6 +239,8 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     [_mediaPlayer addObserver:self forKeyPath:@"time" options:0 context:nil];
     [_mediaPlayer addObserver:self forKeyPath:@"remainingTime" options:0 context:nil];
 
+    [_mediaPlayer setRendererItem:_renderer];
+
     [_listPlayer playItemAtNumber:@(_itemInMediaListToBePlayedFirst)];
 
     if ([self.delegate respondsToSelector:@selector(prepareForMediaPlayback:)])
@@ -294,14 +297,21 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     _playerIsSetup = NO;
     [_shuffleStack removeAllObjects];
 
+    if (@available(iOS 10, *)) {
+        if (_errorCallback && _mediaPlayer.state == VLCMediaPlayerStateError &&  !_sessionWillRestart)
+            [[UIApplication sharedApplication] openURL:_errorCallback options:@{} completionHandler:nil];
+        else if (_successCallback && !_sessionWillRestart)
+            [[UIApplication sharedApplication] openURL:_successCallback options:@{} completionHandler:nil];
+    } else {
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
-    /* UIApplication's replacement calls require iOS 10 or later, which we can't enforce as of yet */
-    if (_errorCallback && _mediaPlayer.state == VLCMediaPlayerStateError &&  !_sessionWillRestart)
-        [[UIApplication sharedApplication] openURL:_errorCallback];
-    else if (_successCallback && !_sessionWillRestart)
-        [[UIApplication sharedApplication] openURL:_successCallback];
+        /* UIApplication's replacement calls require iOS 10 or later, which we can't enforce as of yet */
+        if (_errorCallback && _mediaPlayer.state == VLCMediaPlayerStateError &&  !_sessionWillRestart)
+            [[UIApplication sharedApplication] openURL:_errorCallback];
+        else if (_successCallback && !_sessionWillRestart)
+            [[UIApplication sharedApplication] openURL:_successCallback];
 #pragma clang diagnostic pop
+    }
 
     [[self remoteControlService] unsubscribeFromRemoteCommands];
 
@@ -493,12 +503,12 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
 
 - (void)setAudioDelay:(float)audioDelay
 {
-    _mediaPlayer.currentAudioPlaybackDelay = 1000000.*audioDelay;
+    _mediaPlayer.currentAudioPlaybackDelay = 1000.*audioDelay;
 }
 
 - (float)audioDelay
 {
-    return _mediaPlayer.currentAudioPlaybackDelay/1000000.;
+    return _mediaPlayer.currentAudioPlaybackDelay/1000.;
 }
 
 - (float)playbackPosition
@@ -513,12 +523,12 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
 
 - (void)setSubtitleDelay:(float)subtitleDeleay
 {
-    _mediaPlayer.currentVideoSubTitleDelay = 1000000.*subtitleDeleay;
+    _mediaPlayer.currentVideoSubTitleDelay = 1000.*subtitleDeleay;
 }
 
 - (float)subtitleDelay
 {
-    return _mediaPlayer.currentVideoSubTitleDelay/1000000.;
+    return _mediaPlayer.currentVideoSubTitleDelay/1000.;
 }
 
 - (float)hue
@@ -858,20 +868,44 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     [_mediaPlayer jumpBackward:interval];
 }
 
-- (NSString *)screenAspectRatio
+- (UIScreen *)currentScreen
 {
-    UIScreen *screen = [[UIDevice currentDevice] VLCHasExternalDisplay] ? [UIScreen screens][1] : [UIScreen mainScreen];
-    return [NSString stringWithFormat:@"%d:%d", (int)screen.bounds.size.width, (int)screen.bounds.size.height];
+    return [[UIDevice currentDevice] VLCHasExternalDisplay] ? [UIScreen screens][1] : [UIScreen mainScreen];
+}
+
+- (void)switchToFillToScreen
+{
+    UIScreen *screen = [self currentScreen];
+    CGSize screenSize = screen.bounds.size;
+
+    CGSize videoSize = _mediaPlayer.videoSize;
+
+    CGFloat ar = videoSize.width / (float)videoSize.height;
+    CGFloat dar = screenSize.width / (float)screenSize.height;
+
+    CGFloat scale;
+
+    if (dar >= ar) {
+        scale = screenSize.width / (float)videoSize.width;
+    } else {
+        scale = screenSize.height / (float)videoSize.height;
+    }
+
+    // Multiplied by screen.scale in consideration of pt to px
+    _mediaPlayer.scaleFactor = scale * screen.scale;
+    _isInFillToScreen = YES;
 }
 
 - (void)switchIPhoneXFullScreen
 {
-    BOOL inFullScreen = _mediaPlayer.videoCropGeometry && [[NSString stringWithUTF8String:_mediaPlayer.videoCropGeometry] isEqualToString:[self screenAspectRatio]];
-    if (inFullScreen) {
+    if (_isInFillToScreen) {
         const char *previousAspectRatio = _currentAspectRatio == VLCAspectRatioDefault ? NULL : [[self stringForAspectRatio:_currentAspectRatio] UTF8String];
         _mediaPlayer.videoAspectRatio = (char *)previousAspectRatio;
+        _mediaPlayer.scaleFactor = 0;
+        _isInFillToScreen = NO;
+    } else {
+        [self switchToFillToScreen];
     }
-    _mediaPlayer.videoCropGeometry = inFullScreen ? NULL : (char *)[[self screenAspectRatio] UTF8String];
 }
 
 - (void)switchAspectRatio
@@ -879,15 +913,20 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
     _currentAspectRatio = _currentAspectRatio == VLCAspectRatioSixteenToTen ? VLCAspectRatioDefault : _currentAspectRatio + 1;
     switch (_currentAspectRatio) {
         case VLCAspectRatioDefault:
+            _mediaPlayer.scaleFactor = 0;
             _mediaPlayer.videoAspectRatio = NULL;
             _mediaPlayer.videoCropGeometry = NULL;
             break;
         case VLCAspectRatioFillToScreen:
-            _mediaPlayer.videoCropGeometry = (char *)[[self screenAspectRatio] UTF8String];
+            // Reset aspect ratio only with aspectRatio button since we want to keep
+            // the user ratio with double tap.
+            _mediaPlayer.videoAspectRatio = NULL;
+            [self switchToFillToScreen];
             break;
         case VLCAspectRatioFourToThree:
         case VLCAspectRatioSixteenToTen:
         case VLCAspectRatioSixteenToNine:
+            _mediaPlayer.scaleFactor = 0;
             _mediaPlayer.videoCropGeometry = NULL;
             _mediaPlayer.videoAspectRatio = (char *)[[self stringForAspectRatio:_currentAspectRatio] UTF8String];
     }
@@ -958,9 +997,44 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
 #if !TARGET_OS_TV
 - (BOOL)updateViewpoint:(CGFloat)yaw pitch:(CGFloat)pitch roll:(CGFloat)roll fov:(CGFloat)fov absolute:(BOOL)absolute
 {
+    //adjusting the values
+    if (fabs(yaw) > 180) {
+        yaw = yaw > 0 ? yaw - 360 : yaw + 360;
+    }
+    if (fabs(roll) > 180) {
+        roll = roll > 0 ? roll - 360 : roll + 360;
+    }
+    if (fabs(pitch) > 90) {
+        pitch = pitch > 0 ? pitch - 180 : pitch + 180;
+    }
     return [_mediaPlayer updateViewpoint:yaw pitch:pitch roll:roll fov:fov absolute:absolute];
 }
 
+- (CGFloat)yaw
+{
+    return _mediaPlayer.yaw;
+}
+
+- (CGFloat)pitch
+{
+    return _mediaPlayer.pitch;
+}
+
+- (CGFloat)roll
+{
+    return _mediaPlayer.roll;
+}
+
+- (CGFloat)fov
+{
+    return _mediaPlayer.fov;
+}
+
+- (BOOL)currentMediaIs360Video
+{
+    return [self currentMediaProjection] == VLCMediaProjectionEquiRectangular;
+}
+
 - (NSInteger)currentMediaProjection
 {
     VLCMedia *media = [_mediaPlayer media];
@@ -1190,7 +1264,7 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
 {
     _preBackgroundWrapperView = _videoOutputViewWrapper;
 
-    if (_mediaPlayer.audioTrackIndexes.count > 0)
+    if (!_renderer && _mediaPlayer.audioTrackIndexes.count > 0)
         [self setVideoTrackEnabled:false];
 }
 
@@ -1281,4 +1355,11 @@ typedef NS_ENUM(NSUInteger, VLCAspectRatio) {
               kVLCSettingHardwareDecoding : [defaults objectForKey:kVLCSettingHardwareDecoding]};
 }
 
+#pragma mark - Renderer
+- (void)setRenderer:(VLCRendererItem *)renderer
+{
+    _renderer = renderer;
+    [_mediaPlayer setRendererItem:_renderer];
+}
+
 @end
