@@ -10,8 +10,9 @@
  *
  * Refer to the COPYING file of the official project for license.
  *****************************************************************************/
+#import <UIKit/UIKit.h>
 
-#import <Foundation/Foundation.h>
+static NSString *kPlaylistCellIdentifier = @"PlaylistCell";
 
 @class VLCLinearProgressIndicator;
 @interface VLCPlaylistTableViewCell : UITableViewCell
@@ -28,7 +29,6 @@
 
 @property (nonatomic, strong) NSManagedObject *mediaObject;
 
-+ (VLCPlaylistTableViewCell *)cellWithReuseIdentifier:(NSString *)ident;
 + (CGFloat)heightOfCell;
 
 - (void)collapsWithAnimation:(BOOL)animate;
